class Service:
    """ Services provide methods that have different implementors.
        You can switch between implementors on the fly, which is why this
        is different than using inheritance and overloading. 
        
        For example, you can define a Service that has a multiplyByTwo(x)
        method, and then have two implementors - one that returns
        x * 2, and one that returns, x + x.
    """
        
    def __init__(self, methods):
        """ methods is a list of strings naming all methods the Service should provide 
         e.g. ['copy', 'paste'] """
        self._methods = methods
        self._implementors = {}
        self._descriptions = {}
        self._implementorName = None
        
    def register(self, id, Implementor):
        """ add an implementor """
        implementors = self._implementors
        descriptions = self._descriptions
        methods = list(self._methods)
        
        implementors[id] = {}
        for methodName in methods:
            implementors[id][methodName] = getattr(Implementor, methodName)
        
        descriptions[id] = Implementor.__doc__
        
        self._implementors = implementors
        self._descriptions = descriptions
            
    def listImplementors(self):
        """ return dictionary listing implementors and their descriptions """
        return self._descriptions
        
    def setImplementor(self, implementorName):
        """ set implementor used by the service """
        methodsDict = self._implementors[implementorName]
        for methodName in self._methods:
            setattr(self, methodName, methodsDict[methodName]) 
        self._implementorName = implementorName
    
    def getImplementor(self):
        """ return name of current implementor """
        return self._implementorName
        
